package prt2.breakthrough;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Random;

public class BreakthroughField {

    private int width, height;
    private ArrayList<Coordinate>[] tiles; //tiles[0] - comp, tiles[1] - human
    private final static int COMPUTER = 0;
    private final static int HUMAN = 1;
    public static final int UP = 0;
    public static final int LEFT = -1;
    public static final int RIGHT = +1;

    public void newGame(int width, int height) {
        this.height = height;
        this.width = width;
        tiles = new ArrayList[2];
        tiles[COMPUTER] = new ArrayList<Coordinate>();
        tiles[HUMAN] = new ArrayList<Coordinate>();
        for (int i = 0; i < width; i++) {
            for (int j = 0; j < 2; j++) {
                tiles[COMPUTER].add(new Coordinate(i, j));
                tiles[HUMAN].add(new Coordinate(i, height - 1 - j));
            }
        }
    }

    public Iterator<Coordinate> getComputerTiles() {
        return getTiles(COMPUTER);
    }

    public Iterator<Coordinate> getHumanTiles() {
        return getTiles(HUMAN);
    }

    private Iterator<Coordinate> getTiles(int playerIdx) {
        return tiles[playerIdx].iterator();
    }

    public Coordinate getRandomHumanTile(Random rnd) {
        return getRandomPlayerTile(rnd, HUMAN);
    }

    private Coordinate getRandomComputerTile(Random rnd) {
        return getRandomPlayerTile(rnd, COMPUTER);
    }

    private Coordinate getRandomPlayerTile(Random rnd, int playerIdx) {
        if (tiles[playerIdx].isEmpty()) {
            return null;
        }
        return tiles[playerIdx].get(rnd.nextInt(tiles[playerIdx].size())).clone();
    }

    public Coordinate getPreviousHumanTile(Coordinate tile) {
        return getNeighbourHumanTile(tile, -1);
    }

    public Coordinate getNextHumanTile(Coordinate tile) {
        return getNeighbourHumanTile(tile, +1);
    }

    private Coordinate getNeighbourHumanTile(Coordinate tile, int direction) {
        int idx = 0;
        int humanTilesSize = tiles[HUMAN].size();
        for (int i = 0; i < humanTilesSize; i++) {
            Coordinate humanTile = tiles[HUMAN].get(i);
            if (humanTile.equals(tile)) {
                idx = i;
                break;
            }
        }
        idx += direction;
        idx += humanTilesSize;
        idx %= humanTilesSize;
        return tiles[HUMAN].get(idx).clone();
    }

    public boolean move(Coordinate tile, int dir) {
        boolean humanMoveSuccess = move(tile, dir, HUMAN);
        if (!humanMoveSuccess) {
            return false;
        } else {
            if (humanWon()) {
                return true;
            }
            while (!moveEnemy()) {
            }
            return true;
        }
    }

    private boolean move(Coordinate tile, int dir, int playerIdx) {
        for (Coordinate playerTile : tiles[playerIdx]) {
            if (playerTile.equals(tile)) {
                Coordinate clone = tile.clone();
                clone.x += dir;
                if (clone.x < 0) {
                    clone = tile.clone();
                } else if (clone.x == width) {
                    clone = tile.clone();
                } else if (playerIdx == HUMAN) {
                    clone.y--;
                } else {
                    clone.y++;
                }
                for (Coordinate coordinate : tiles[playerIdx]) {
                    if (clone.equals(coordinate)) {
                        return false;
                    }
                }
                Coordinate enemyHitTile = null;
                for (Coordinate coordinate : tiles[1 - playerIdx]) {
                    if (clone.equals(coordinate)) {
                        enemyHitTile = coordinate;
                        break;
                    }
                }
                if (enemyHitTile != null) {
                    if (dir != UP) {
                        tiles[1 - playerIdx].remove(enemyHitTile);
                    } else {
                        return false;
                    }
                }
                playerTile.x = clone.x;
                playerTile.y = clone.y;
                return true;
            }
        }
        return false;
    }

    private boolean moveEnemy() {
        Random random = new Random();
        return move(getRandomComputerTile(random), random.nextInt(3) - 1, COMPUTER);
    }

    public boolean humanWon() {
        return playerWon(HUMAN);
    }

    public boolean computerWon() {
        return playerWon(COMPUTER);
    }

    private boolean playerWon(int playerIdx) {
        if (tiles[1 - playerIdx].isEmpty()) {
            return true;
        }
        for (Coordinate coordinate : tiles[playerIdx]) {
            if (coordinate.y == (playerIdx == HUMAN ? 0 : height - 1)) {
                return true;
            }
        }
        return false;
    }

    public boolean hasHumanTileAt(Coordinate coordinate) {
        for (Coordinate coordinate1 : tiles[HUMAN]) {
            if (coordinate.equals(coordinate1)) {
                return true;
            }
        }
        return false;
    }
}
