package prt2.breakthrough;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.res.Resources;
import android.os.Bundle;
import android.util.AttributeSet;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.widget.TextView;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Random;

public class GameView extends TileView {

    private int mode = READY;
    private int oldMode = READY;
    public static final int PAUSE = 0;
    public static final int READY = 1;
    public static final int RUNNING_SELECT = 2;
    public static final int RUNNING_MOVE = 3;
    public static final int LOSE = 4;
    public static final int WIN = 5;
    private TextView statusText;
    private BreakthroughField bf;
    private static final int COMP_TILE = 1;
    private static final int HUMAN_TILE = 2;
    private static final int SELECTED_TILE = 3;
    private static final int MOVED_TILE = 4;
    private Coordinate selectedHumanTile, movedHumanTile;

    public GameView(Context context, AttributeSet attrs) {
        super(context, attrs);
        initGameView();
    }

    public GameView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        initGameView();
    }

    private void initGameView() {
        setFocusable(true);
        Resources r = getContext().getResources();
        int tileCount = 5;
        resetTiles(tileCount);
        loadTile(COMP_TILE, r.getDrawable(R.drawable.redstar));
        loadTile(HUMAN_TILE, r.getDrawable(R.drawable.greenstar));
        loadTile(SELECTED_TILE, r.getDrawable(R.drawable.greystar));
        loadTile(MOVED_TILE, r.getDrawable(R.drawable.yellowstar));
    }

    private void initNewGame() {
        bf = new BreakthroughField();
        bf.newGame(xTileCount, yTileCount);
        obtainSelectedHumanTile();
    }

    public Bundle saveState() {
        Bundle map = new Bundle();
//        map.putInt("someInteger", Integer.valueOf(someInteger));
//        map.putIntArray("someArray", coordArrayListToArray(someArray));
        return map;
    }

    public void restoreState(Bundle icicle) {
        setMode(PAUSE);
//        someArray = coordArrayToArrayList(icicle.getIntArray("someArray"));
//        someInteger = icicle.getInt("someInteger");
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent msg) {
        switch (keyCode) {
            case KeyEvent.KEYCODE_P:
                setMode(PAUSE);
                return true;
            case KeyEvent.KEYCODE_DPAD_UP:
            case KeyEvent.KEYCODE_W:
                if (mode == READY || mode == LOSE || mode == WIN) {
                    initNewGame();
                    setMode(RUNNING_SELECT);
                    return true;
                }
                if (mode == PAUSE) {
                    setMode(oldMode);
                    update();
                    return true;
                }
                if (mode == RUNNING_SELECT) {
                    setMode(RUNNING_MOVE);
                    update();
                    return true;
                }
                if (mode == RUNNING_MOVE && movedHumanTile.y > 0) {
                    move(BreakthroughField.UP);
                }
                return true;
            case KeyEvent.KEYCODE_DPAD_DOWN:
            case KeyEvent.KEYCODE_S:
                if (mode == RUNNING_MOVE) {
                    setMode(RUNNING_SELECT);
                    update();
                }
                return true;
            case KeyEvent.KEYCODE_DPAD_LEFT:
            case KeyEvent.KEYCODE_A:
                if (mode == RUNNING_SELECT) {
                    selectedHumanTile = bf.getPreviousHumanTile(selectedHumanTile);
                    movedHumanTile = selectedHumanTile.clone();
                    update();
                } else {
                    move(BreakthroughField.LEFT);
                }
                return true;
            case KeyEvent.KEYCODE_DPAD_RIGHT:
            case KeyEvent.KEYCODE_D:
                if (mode == RUNNING_SELECT) {
                    selectedHumanTile = bf.getNextHumanTile(selectedHumanTile);
                    movedHumanTile = selectedHumanTile.clone();
                    update();
                } else {
                    move(BreakthroughField.RIGHT);
                }
                return true;
            default:
                return super.onKeyDown(keyCode, msg);
        }
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        if (mode != RUNNING_SELECT && mode != RUNNING_MOVE) {
            return super.onTouchEvent(event);
        }
        float tx = event.getAxisValue(MotionEvent.AXIS_X);
        float ty = event.getAxisValue(MotionEvent.AXIS_Y);
        Coordinate coordinate = new Coordinate((int) ((tx - xOffset) / tileSize), (int) ((ty - yOffset) / tileSize));
        if (bf.hasHumanTileAt(coordinate)) {
            selectedHumanTile = coordinate.clone();
            movedHumanTile = selectedHumanTile.clone();
            setMode(RUNNING_MOVE);
            update();
        } else if (mode == RUNNING_MOVE && coordinate.y + 1 == selectedHumanTile.y) {
            move(coordinate.x - selectedHumanTile.x);
        }
        return super.onTouchEvent(event);
    }

    public void setTextView(TextView newView) {
        statusText = newView;
    }

    public void setMode(int newMode) {
        oldMode = mode;
        mode = newMode;
        if (newMode == RUNNING_SELECT && oldMode != RUNNING_SELECT && oldMode != RUNNING_MOVE) {
            statusText.setVisibility(View.INVISIBLE);
            update();
            showHelp();
            return;
        }
        Resources res = getContext().getResources();
        CharSequence str = "";
        if (newMode == PAUSE) {
            str = res.getText(R.string.mode_pause);
        }
        if (newMode == READY) {
            str = res.getText(R.string.mode_ready);
        }
        if (newMode == LOSE) {
            str = res.getString(R.string.mode_lose_prefix)
                    + res.getString(R.string.mode_cont_suffix);
        }
        if (newMode == WIN) {
            str = res.getString(R.string.mode_win_prefix)
                    + res.getString(R.string.mode_cont_suffix);
        }
        statusText.setText(str);
        statusText.setVisibility(View.VISIBLE);
    }

    public void update() {
        if (mode == RUNNING_SELECT || mode == RUNNING_MOVE) {
            clearTiles();
            drawTiles();
            invalidate();
        }
    }

    private void drawTiles() {
        for (Iterator<Coordinate> it = bf.getComputerTiles(); it.hasNext();) {
            Coordinate coord = it.next();
            setTile(COMP_TILE, coord.x, coord.y);
        }
        for (Iterator<Coordinate> it = bf.getHumanTiles(); it.hasNext();) {
            Coordinate coord = it.next();
            if (!coord.equals(selectedHumanTile)) {
                setTile(HUMAN_TILE, coord.x, coord.y);
            }
        }
        setTile(SELECTED_TILE, selectedHumanTile.x, selectedHumanTile.y);
        if (mode == RUNNING_MOVE) {
            setTile(MOVED_TILE, movedHumanTile.x, movedHumanTile.y);
        }
    }

    private void showHelp() {
        new AlertDialog.Builder(getContext()).setTitle("Help").setMessage(getResources().getText(R.string.help_text)).setPositiveButton("OK", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                dialog.dismiss();
            }
        }).create().show();
    }

    private void move(int dir) {
        if (bf.move(movedHumanTile, dir)) {
            if (bf.humanWon()) {
                update();
                setMode(WIN);
            } else if (bf.computerWon()) {
                update();
                setMode(LOSE);
            } else {
                setMode(RUNNING_SELECT);
                obtainSelectedHumanTile();
                update();
            }
        }
    }

    private void obtainSelectedHumanTile() {
        selectedHumanTile = bf.getRandomHumanTile(RND);
        movedHumanTile = selectedHumanTile.clone();
    }
}