package prt2.breakthrough;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.drawable.Drawable;
import android.util.AttributeSet;
import android.view.View;
import java.util.ArrayList;
import java.util.Random;

public class TileView extends View {
    protected static final Random RND = new Random();

    protected int tileSize;
    protected int xTileCount;
    protected int yTileCount;
    protected int xOffset;
    protected int yOffset;
    private Bitmap[] tileArray;
    private int[][] tileGrid;
    private final Paint paint = new Paint();

    public TileView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        tileSize = -1;
        readAttrs(attrs);
    }

    public TileView(Context context, AttributeSet attrs) {
        super(context, attrs);
        readAttrs(attrs);
    }

    public void resetTiles(int tilecount) {
        tileArray = new Bitmap[tilecount];
    }

    @Override
    protected void onSizeChanged(int w, int h, int oldw, int oldh) {
        xTileCount = (int) Math.floor(w / tileSize);
        yTileCount = (int) Math.floor(h / tileSize);

        xOffset = ((w - (tileSize * xTileCount)) / 2);
        yOffset = ((h - (tileSize * yTileCount)) / 2);

        tileGrid = new int[xTileCount][yTileCount];
        clearTiles();
    }

    public void loadTile(int index, Drawable tile) {
        Bitmap bitmap = Bitmap.createBitmap(tileSize, tileSize, Bitmap.Config.ARGB_8888);
        Canvas canvas = new Canvas(bitmap);
        tile.setBounds(0, 0, tileSize, tileSize);
        tile.draw(canvas);

        tileArray[index] = bitmap;
    }

    public void clearTiles() {
        for (int x = 0; x < xTileCount; x++) {
            for (int y = 0; y < yTileCount; y++) {
                setTile(0, x, y);
            }
        }
    }

    public void setTile(int tileindex, int x, int y) {
        tileGrid[x][y] = tileindex;
    }

    @Override
    public void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        for (int x = 0; x < xTileCount; x += 1) {
            for (int y = 0; y < yTileCount; y += 1) {
                if (tileGrid[x][y] > 0) {
                    canvas.drawBitmap(tileArray[tileGrid[x][y]],
                            xOffset + x * tileSize,
                            yOffset + y * tileSize,
                            paint);
                }
            }
        }

    }

    private void readAttrs(AttributeSet attrs) {
        try {
            for (int i = 0; i < attrs.getAttributeCount(); i++) {
                if (attrs.getAttributeName(i).equals("tileSize")) {
                    tileSize = Integer.parseInt(attrs.getAttributeValue(i));
                }
            }
            if (tileSize == -1) {
                throw new NumberFormatException();
            }
        } catch (NumberFormatException ex) {
            tileSize = 12;
        }
    }

    protected int[] coordArrayListToArray(ArrayList<Coordinate> cvec) {
        int count = cvec.size();
        int[] rawArray = new int[count * 2];
        for (int index = 0; index < count; index++) {
            Coordinate c = cvec.get(index);
            rawArray[2 * index] = c.x;
            rawArray[2 * index + 1] = c.y;
        }
        return rawArray;
    }

    protected ArrayList<Coordinate> coordArrayToArrayList(int[] rawArray) {
        ArrayList<Coordinate> coordArrayList = new ArrayList<Coordinate>();
        int coordCount = rawArray.length;
        for (int index = 0; index < coordCount; index += 2) {
            Coordinate c = new Coordinate(rawArray[index], rawArray[index + 1]);
            coordArrayList.add(c);
        }
        return coordArrayList;
    }

    protected Coordinate randomCoordinate() {
        return new Coordinate(GameView.RND.nextInt(xTileCount), GameView.RND.nextInt(yTileCount));
    }
}