package prt2.game;

import android.content.Context;
import android.content.res.Resources;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.AttributeSet;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.widget.TextView;
import java.util.ArrayList;
import java.util.Random;

public class GameView extends TileView {

    private int mode = READY;
    public static final int PAUSE = 0;
    public static final int READY = 1;
    public static final int RUNNING = 2;
    public static final int LOSE = 3;
    public static final int WIN = 4;
    private TextView statusText;
    private static final Random RND = new Random();
    private RefreshHandler redrawHandler = new RefreshHandler();
    private long lastMove;

    
    private Coordinate randomCoordinate() {
        return new Coordinate(RND.nextInt(xTileCount), RND.nextInt(yTileCount));
    }

    class RefreshHandler extends Handler {

        @Override
        public void handleMessage(Message msg) {
            GameView.this.update();
            GameView.this.invalidate();
        }

        public void sleep(long delayMillis) {
            removeMessages(0);
            sendMessageDelayed(obtainMessage(0), delayMillis);
        }
    };

    public GameView(Context context, AttributeSet attrs) {
        super(context, attrs);
        initGameView();
    }

    public GameView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        initGameView();
    }

    private void initGameView() {
        setFocusable(true);
        Resources r = getContext().getResources();
        int tileCount = 0;
        resetTiles(tileCount);
//        loadTile(XX, r.getDrawable(R.drawable.xx));
    }

    private void initNewGame() {
    }

    private int[] coordArrayListToArray(ArrayList<Coordinate> cvec) {
        int count = cvec.size();
        int[] rawArray = new int[count * 2];
        for (int index = 0; index < count; index++) {
            Coordinate c = cvec.get(index);
            rawArray[2 * index] = c.x;
            rawArray[2 * index + 1] = c.y;
        }
        return rawArray;
    }

    public Bundle saveState() {
        Bundle map = new Bundle();
//        map.putInt("someInteger", Integer.valueOf(someInteger));
//        map.putIntArray("someArray", coordArrayListToArray(someArray));
        return map;
    }

    private ArrayList<Coordinate> coordArrayToArrayList(int[] rawArray) {
        ArrayList<Coordinate> coordArrayList = new ArrayList<Coordinate>();
        int coordCount = rawArray.length;
        for (int index = 0; index < coordCount; index += 2) {
            Coordinate c = new Coordinate(rawArray[index], rawArray[index + 1]);
            coordArrayList.add(c);
        }
        return coordArrayList;
    }

    public void restoreState(Bundle icicle) {
        setMode(PAUSE);
//        someArray = coordArrayToArrayList(icicle.getIntArray("someArray"));
//        someInteger = icicle.getInt("someInteger");
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent msg) {
        if (keyCode == KeyEvent.KEYCODE_P) {
            setMode(PAUSE);
            return true;
        }
        if (keyCode == KeyEvent.KEYCODE_DPAD_UP) {
            if (mode == READY || mode == LOSE || mode == WIN) {
                initNewGame();
                setMode(RUNNING);
                update();
                return true;
            }
            if (mode == PAUSE) {
                setMode(RUNNING);
                update();
                return true;
            }

            return true;
        }

        if (keyCode == KeyEvent.KEYCODE_DPAD_DOWN) {

            return true;
        }

        if (keyCode == KeyEvent.KEYCODE_DPAD_LEFT) {

            return true;
        }

        if (keyCode == KeyEvent.KEYCODE_DPAD_RIGHT) {

            return true;
        }

        return super.onKeyDown(keyCode, msg);
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        if (mode != RUNNING) {
            return super.onTouchEvent(event);
        }
        float tx = event.getAxisValue(MotionEvent.AXIS_X);
        float ty = event.getAxisValue(MotionEvent.AXIS_Y);

        return super.onTouchEvent(event);
    }

    public void setTextView(TextView newView) {
        statusText = newView;
    }

    public void setMode(int newMode) {
        int oldMode = mode;
        mode = newMode;
        if (newMode == RUNNING && oldMode != RUNNING) {
            statusText.setVisibility(View.INVISIBLE);
            update();
            return;
        }
        Resources res = getContext().getResources();
        CharSequence str = "";
        if (newMode == PAUSE) {
            str = res.getText(R.string.mode_pause);
        }
        if (newMode == READY) {
            str = res.getText(R.string.mode_ready);
        }
        if (newMode == LOSE) {
            str = res.getString(R.string.mode_lose_prefix)
                    + res.getString(R.string.mode_cont_suffix);
        }
        if (newMode == WIN) {
            str = res.getString(R.string.mode_win_prefix)
                    + res.getString(R.string.mode_cont_suffix);
        }
        statusText.setText(str);
        statusText.setVisibility(View.VISIBLE);
    }

    public void update() {
        long moveDelay = 600;
        if (mode == RUNNING) {
            long now = System.currentTimeMillis();

            if (now - lastMove > moveDelay) {
                clearTiles();
//                drawSomething();
                lastMove = now;
            }
            redrawHandler.sleep(moveDelay);
        }
    }

    private class Coordinate {

        public int x;
        public int y;

        public Coordinate(int newX, int newY) {
            x = newX;
            y = newY;
        }

        public boolean equals(Coordinate other) {
            if (x == other.x && y == other.y) {
                return true;
            }
            return false;
        }
    }
}