package prt2.game;

import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.drawable.Drawable;
import android.util.AttributeSet;
import android.view.View;

public class TileView extends View {

    protected int tileSize;
    protected int xTileCount;
    protected int yTileCount;
    private int xOffset;
    private int yOffset;
    private Bitmap[] tileArray;
    private int[][] tileGrid;
    private final Paint paint = new Paint();

    public TileView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        TypedArray a = context.obtainStyledAttributes(attrs, R.styleable.TileView);

        tileSize = a.getInt(R.styleable.TileView_tileSize, 12);

        a.recycle();
    }

    public TileView(Context context, AttributeSet attrs) {
        super(context, attrs);

        TypedArray a = context.obtainStyledAttributes(attrs, R.styleable.TileView);

        tileSize = a.getInt(R.styleable.TileView_tileSize, 12);

        a.recycle();
    }

    public void resetTiles(int tilecount) {
        tileArray = new Bitmap[tilecount];
    }

    @Override
    protected void onSizeChanged(int w, int h, int oldw, int oldh) {
        xTileCount = (int) Math.floor(w / tileSize);
        yTileCount = (int) Math.floor(h / tileSize);

        xOffset = ((w - (tileSize * xTileCount)) / 2);
        yOffset = ((h - (tileSize * yTileCount)) / 2);

        tileGrid = new int[xTileCount][yTileCount];
        clearTiles();
    }

    public void loadTile(int index, Drawable tile) {
        Bitmap bitmap = Bitmap.createBitmap(tileSize, tileSize, Bitmap.Config.ARGB_8888);
        Canvas canvas = new Canvas(bitmap);
        tile.setBounds(0, 0, tileSize, tileSize);
        tile.draw(canvas);

        tileArray[index] = bitmap;
    }

    public void clearTiles() {
        for (int x = 0; x < xTileCount; x++) {
            for (int y = 0; y < yTileCount; y++) {
                setTile(0, x, y);
            }
        }
    }

    public void setTile(int tileindex, int x, int y) {
        tileGrid[x][y] = tileindex;
    }

    @Override
    public void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        for (int x = 0; x < xTileCount; x += 1) {
            for (int y = 0; y < yTileCount; y += 1) {
                if (tileGrid[x][y] > 0) {
                    canvas.drawBitmap(tileArray[tileGrid[x][y]],
                            xOffset + x * tileSize,
                            yOffset + y * tileSize,
                            paint);
                }
            }
        }

    }
}